/*
 * RandomCsp4C - A base class for a generator for random binary symmetric constraint satisfaction problems.
 * Part of the RandomCsp library.
 * Copyright (C) 1998-2000 by J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * $Id: randomcsp4.h,v 1.3 2004/02/16 16:06:53 jvhemert Exp $
 *
 */


#ifndef _RandomCsp4C_H
#define _RandomCsp4C_H


#include <cstdlib>
#include "randomcsp.h"


/**
 * These types can and should be used as a type for variables in your own code.
 *
 * @name Type definitions for RandomCsp4C
 *
 * @memo General type definitons
 *
 */

//@{
/// Determines the number of conflicts a constraint contains.
typedef double TightnessT;

/// Determines the number of nodes in the network.
typedef double ConnectivityT;
//@}



/**
 * This class is a empty class that can be extended to a class capable of
 * producing Random Binary Constraint Satisfaction Problems. The class should
 * implement the ProduceProblem function. This base class provides an interface
 * to models that use four parameters to describe instances: number of
 * variables, domain size, tightness of constraints and connectivity (density
 * of constraints). Also provided is a parameter to ask for random domain sizes
 * over the variables.
 *
 * @memo Random Constraint Satisfaction Problems
 *
 * @author J.I. van Hemert @version 1.2
 *
 */

class RandomCsp4C: 
	public RandomCspA
{
	public:



		/**
		 * Constructor copies all the parameters for further usage.
		 *
		 * @memo Constructor
		 *
		 * @param iNumberOfVariables Number of variables in CSP.
		 * @param iDomainSize Size of domain of each variable *or* maximum size of domain.
		 * @param iRandomDomainSize If true than a the domainsize will be varied over the variables.
		 * @param iTightness Determines percentage of conflicts that are generated.
		 * @param iConnectivity Percentage of constraints that are generated.
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		RandomCsp4C
		(
			NumberOfVariablesT const iNumberOfVariables,
			DomainSizeT const iDomainSize,
			bool const iRandomDomainSize,
			TightnessT const iTightness,
			ConnectivityT const iConnectivity
		);



		/**
		 * For C++'s sake we have to implement the virtual destructor.
		 *
		 * @memo Destructor
		 *
		 * @return nothing.
		 *
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		~RandomCsp4C ();



		/**
		 * Create a filename from all the parameters of the CSP.
		 *
		 * @memo Producing a filename
		 *
		 * @param FileName reference to a string (char *) that should have
		 * enough memory for holding the filename produced by this function,
		 * maximum required is 100 bytes.
		 *
		 * @return nothing (FileName is the returnvalue).
		 *
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		void ProduceFileName
		(
			char * FileName
		);



		/**
		 * Produce a constraint satisfaction method using the method of Patrick
		 * Prosser, which he mentiones in his article 'Binary Constraint
		 * Satisfaction Problems: Some are Harder than Others' in '11th
		 * European Conference on Artificial Inteligence (ECAI94)' edited by A.
		 * Cohn and published in 1994 by * John Wiley \& Sons, Ltd.
		 *
		 * @memo Producing a Random CSP
		 *
		 * @return nothing.
		 * 
		 * @author J.I. van Hemert
		 * @version 1.2
		 *
		 */

		virtual void const ProduceProblem ();

		bool const IsRandomDomainSize () const
		{
			return RandomDomainSize;
		};

		ConnectivityT const GetConnectivity () const
		{
			return Connectivity;
		};

		TightnessT const GetTightness () const
		{
			return Tightness;
		};

		DomainSizeT const GetGlobalDomainSize () const
		{
			return DomainSize;
		};

	protected:

	private:
		// Whether or not to use random sizes for the domains of the variables.
		bool RandomDomainSize;

		// Global size of domain of the variables if RandomDomainSize=false
		// *or*
		// values lie between 0..DomainSize-1 in case of RandomDomainSize=true.
		DomainSizeT DomainSize;

		// The tightness gives the probability a conflict is produced.
		TightnessT Tightness;

		// The connectivity gives the number of generated constraints.
		ConnectivityT Connectivity;

}; // RandomCsp4C


#endif

// eof pprosser.h
